"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.iter = void 0;
const HIterator_1 = require("./HIterator");
// Main functiob
const iter = function iter(item) {
    return HIterator_1.HIterator.from(item);
};
exports.iter = iter;
// STATIC HELPERS - ITERATOR CREATOR
exports.iter.range = (start, stop, step) => {
    if (stop === undefined) {
        stop = start;
        start = 0;
        step = step === undefined ? (stop >= 0 ? 1 : -1) : step;
    }
    if (step === undefined) {
        // start and stop are defined
        step = start <= stop ? 1 : -1;
    }
    let current = start, end = stop, toAdd = step;
    return HIterator_1.HIterator.from({
        next() {
            const currentValue = current;
            const hasEnded = toAdd < 0 ? currentValue <= end : currentValue >= end;
            if (hasEnded) {
                return { value: undefined, done: true };
            }
            else {
                current += toAdd;
                return { value: currentValue, done: false };
            }
        },
    });
};
exports.iter.repeat = (item, times) => {
    let iterations = 0;
    let repetitions = times === undefined ? Infinity : times;
    if (repetitions < 0 || isNaN(repetitions)) {
        throw new RangeError('Invalid times value for repeat iterator.');
    }
    return HIterator_1.HIterator.from({
        next() {
            if (iterations < repetitions) {
                iterations++;
                return { value: item, done: false };
            }
            return { value: undefined, done: true };
        },
    });
};
